/*
 * Copy me if you can.
 * by 20h
 */

#include <u.h>
#include <libc.h>
#include "xmlpull.h"
#include "rssread.h"

void
freefeed(feed *f)
{
	if(f != nil){
		if(f->t != nil)
			free(f->t);
		if(f->l != nil)
			free(f->l);
		if(f->d != nil)
			free(f->d);
		free(f);
	}
	return;
}

void
freefeedt(feed *r)
{
	while(r != nil){
		if(r->n != nil){
			r = r->n;
			freefeed(r->p);
		} else {
			freefeed(r);
			r = nil;
		}
	}
	return;
}

feed *
removefeed(feed *r, feed *f)
{
	if(f->n != nil && f->p != nil){
		f->n->p = f->p;
		f->p->n = f->n;
	} else {
		if(f->n != nil){
			f->n->p = nil;
			r = f->n;
		}
		if(f->p != nil)
			f->p->n = nil;
	}
	freefeed(f);

	return r;
}

feed *
searchfeed(feed *r, char *t, char *l, char *d)
{
	while(r != nil){
		if(r->t != nil && t != nil){
			if(!strcmp(r->t, t)){
				r->s = 1;
				return r;
			}
		}
		if(r->l != nil && l != nil){
			if(!strcmp(r->l, l)){
				r->s = 1;
				return r;
			}
		}
		if(r->d != nil && d != nil){
			if(!strcmp(r->d, d)){
				r->s = 1;
				return r;
			}
		}
		r = r->n;
	}
	return nil;
}

feed *
addfeed(feed *r, feed *f)
{
	feed *ret;

	ret = r;
	f->s = 2;
	if(r != nil) {
		while(r->n != nil)
			r = r->n;
	} else
		return f;
	r->n = f;
	f->p = r;

	return ret;
}
	
feed *
checkfeed(feed *r)
{
	feed *a;

	a = r;

	while(a != nil){
		if(a->s == 0)
			r = removefeed(r, a);
		else
			a->s = 0;
		a = a->n;
	}

	return r;
}

void
showfeed(feed *r)
{
	if(r != nil){
		while(r->n != nil)
			r = r->n;

		while(r != nil){
			if(r->s == 2){
				if(r->t != nil)
					print("title: %s\n", r->t);
				if(r->l != nil)
					print("link:  %s\n", r->l);
				if(r->d != nil)
					print("%s\n", r->d);
				print("\n");
			}
			r = r->p;
		}
	}

	return;
}

void
main(void)
{
	xmlpull *x, *a;
	char st;
	feed *f, *r;

	st = NONE;
	f = nil;
	r = nil;

	x = openxmlpull(0);
	while((a = nextxmlpull(x)) != nil && st != END){
		switch(a->ev){
		case START_DOCUMENT:
			break;
		case START_TAG:
			if(!strcmp(x->na, "item") || !strcmp(x->na, "entry")){
				if(f != nil)
					freefeed(f);
				f = mallocz(sizeof(feed), 2);
				st = ITEM;
				break;
			}
			if(!strcmp(x->na, "title") && st == ITEM){
				st = TITLE;
				break;
			}
			if(!strcmp(x->na, "link") && st == ITEM){
				st = LINK;
				break;
			}
			if(!strcmp(x->na, "description") && st == ITEM){
				st = DESC;
				break;
			}
			break;
		case START_END_TAG:
			break;
		case ATTR:
			if(!strcmp(x->na, "href") && st == LINK)
				f->l = strdup(x->va);
			break;
		case TEXT:
			switch(st){
			case TITLE:
				f->t = strdup(x->na);
				break;
			case LINK:
				f->l = strdup(x->na);
				break;
			case DESC:
				f->d = strdup(x->na);
				break;
			default:
				break;
			}
			break;
		case END_TAG:
			if((!strcmp(x->na, "item") || !strcmp(x->na, "entry")) && st == ITEM){
				if(searchfeed(r, f->t, f->l, f->d) == nil){
					r = addfeed(r, f);
					f = nil;
				} else {
					freefeed(f);
					f = nil;
				}
							
				st = NONE;
				break;
			}
			if(!strcmp(x->na, "title") && st == TITLE){
				st = ITEM;
				break;
			}
			if(!strcmp(x->na, "link") && st == LINK){
				st = ITEM;
				break;
			}
			if(!strcmp(x->na, "description") && st == DESC){
				st = ITEM;
				break;
			}
			if(!strcmp(x->na, "rdf:RDF") || !strcmp(x->na, "items")
						|| !strcmp(x->na, "rss") || !strcmp(x->na, "feed")){
				showfeed(r);
				r = checkfeed(r);
				break;
			}
			break;
		case END_DOCUMENT:
			st = END;
			break;
		default:
			print("Please report this to the xmlpull author: %x\n", x->ev);
			break;
		}
	}
	freexmlpull(x);

	freefeedt(r);

	exits(0);
}
